/*==========================================================================*/
/*  Galaxy Demo                                                             */
/*  Author: Roman Podobedov                                                 */
/*  Email: romka@ut.ee                                                      */
/*  WEB: http://romka.demonews.com                                          */
/*  Date: 5 October 2000                                                    */
/*==========================================================================*/

#include <math.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include "texture.h"

#define PI 3.14159f
#define GALAXY_P 200
#define GALAXY_P1 800

struct point
{
  float x, y, z;
};
struct g_point
{
	float x, y, z;
	float r, g, b;
	float speed;
};

/* Window parameters */
//char *className = "OpenGL";
//char *windowName = "OpenGL Galaxy Demo";
//int winX = 0, winY = 0;
//int winWidth = 800, winHeight = 600;
//HDC hDC;
//HDC hDCFrontBuffer;
//HGLRC hGLRC;
unsigned *teximage;
GLuint texture[4];     /* Textures */

float ang=0;
struct point cam, upv, camd; /* Camera position, Up vector and camera direction point */

struct g_point p1[GALAXY_P];
struct g_point p2[GALAXY_P];
struct g_point p3[GALAXY_P1];

/*=================== Load Texture =========================================*/
void LoadTexture(char *fn, int t_num)
{
  int texwid, texht;
  int texcomps;
 
  teximage = read_texture(fn, &texwid, &texht, &texcomps);
  if (!teximage) exit(0);
  glBindTexture(GL_TEXTURE_2D, texture[t_num]);
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, texwid, texht, 0, GL_RGBA, GL_UNSIGNED_BYTE, teximage);

  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 

  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

  free(teximage);
} 

/*=================== Initialization =======================================*/
static void Init()
{
  int i, j;
	float alpha, x, z, r, theta, a, b;
	
	/* Load all textures to memory */
  glGenTextures(4, texture); 
  LoadTexture([[[NSBundle bundleForClass:[Galaxy class]] pathForResource:@"particle0" ofType:@"rgb"] cString], 0);
  LoadTexture([[[NSBundle bundleForClass:[Galaxy class]] pathForResource:@"particle1" ofType:@"rgb"] cString], 3);
  LoadTexture([[[NSBundle bundleForClass:[Galaxy class]] pathForResource:@"back" ofType:@"rgb"] cString], 1);
  LoadTexture([[[NSBundle bundleForClass:[Galaxy class]] pathForResource:@"clouds" ofType:@"rgb"] cString], 2);

  /* Enable color blending */
  glBlendFunc(GL_ONE, GL_ONE);
  glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);

  /* Camera properties: Camera Up vector, camera position, camera direction */
	upv.x = 0;
  upv.y = 10;
  upv.z = 0;
  cam.x = 0;
  cam.y = 50;
  cam.z = 40;
	camd.x = 0;
	camd.y = 0;
	camd.z = 0;

	/* Build Galaxy */

  /* Calculate central level (very condensed particles) */
	for (j=0; j<4; j++)
  {
    a = 0.9f;
    b = 0.31f;
    theta = 0;
    for (i=0; i<50; i++)
    {
      /* Spiral formula: r = a*exp(b*theta) */
      r = a*(float)exp(b*theta);
      theta += 2.5f/(float)(i+1);
			/* Particle coordinates in xz plane */
			x = r*(float)cos(theta);
      z = r*(float)sin(theta);

      alpha = (float)(((float)j*90)*PI)/180;

			/* Particle coordinates in xz plane + rotation */
			p1[j*50+i].x = x*(float)cos(alpha)-z*(float)sin(alpha);
			p1[j*50+i].z = x*(float)sin(alpha)+z*(float)cos(alpha);
			/* Particle color */
			x = (float)(rand() % 256)/256;
			p1[j*50+i].r = x;
			p1[j*50+i].g = x;
			p1[j*50+i].b = x;
    }
  }

  /* Next level of particles: normal condensation */
	for (j=0; j<4; j++)
  {
    a = 1.9f;
    b = 0.3f;
    theta = 0;
    for (i=0; i<50; i++)
    {
      /* r = a*exp(b*theta) */
      r = a*(float)exp(b*theta);
      theta += 2.3f/(float)(i+1);
      x = r*(float)cos(theta);
      z = r*(float)sin(theta);

      alpha = (((float)j*90+(float)(rand() % 180))*PI)/180;

			/* Particle coordinates in xyz space + rotation */
			p2[j*50+i].x = x*(float)cos(alpha)-z*(float)sin(alpha);
			p2[j*50+i].y = (float)(rand() % 40)-20;
			p2[j*50+i].z = x*(float)sin(alpha)+z*(float)cos(alpha);
			
			/* Particle color */
			x = (float)(rand() % 256)/256;
			p2[j*50+i].r = 0.2f+x;
			p2[j*50+i].g = 0.2f+x;
			p2[j*50+i].b = 0.5f+x;

			/* Random speed of particles */
			p2[j*50+i].speed = (float)(rand() % 100)/100+0.1f;
    }
  }

  for (j=0; j<4; j++)
  {
    a = 1.5f;
    b = 0.3f;
    theta = 0;
    for (i=0; i<200; i++)
    {
      /* r = a*exp(b*theta) */
      r = a*(float)exp(b*theta);
      theta += 2.2f/(float)(i+1);
      x = r*(float)cos(theta);
      z = r*(float)sin(theta);

      alpha = (float)(((float)j*90+(float)(rand() % 360))*PI)/180;

			/* Particle coordinates in xyz space + rotation */
			p3[j*50+i].x = x*(float)cos(alpha)-z*(float)sin(alpha);
			p3[j*50+i].y = (float)(rand() % 40)-20;
			p3[j*50+i].z = x*(float)sin(alpha)+z*(float)cos(alpha);

			/* Particle color */
			x = (float)(rand() % 256)/256;
			p3[j*50+i].r = 0.5f+x;
			p3[j*50+i].g = 0.5f+x;
			p3[j*50+i].b = 0.2f+x;

			/* Random speed of particles */
			p3[j*50+i].speed = (float)(rand() % 100)/100+0.1f;
    }
  }
}

/*=================== Projection settings ==================================*/
static void setProjection(void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-10.6666, 10.6666, -10.5, 10.5, 10, 150);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

/*=================== Window resizing routine ==============================*/
static void resize(int width, int height)
{
    glViewport(0, 0, width, height);
    setProjection();
}

/*=================== Vector Length calculation ============================*/
static float VectorLen(struct point *v)
{
	return (float)sqrt(v->x*v->x + v->y*v->y + v->z*v->z);
}

/*=================== Vector scalar multiplication =========================*/
static float ScalarMult(struct point *v1, struct point *v2)
{
	return v1->x*v2->x + v1->y*v2->y + v1->z*v1->z;
}

/*================== Vector normalization ==================================*/
void normalize(struct point *V)
{
  float d;

  /* Vector length */
  d = (float)sqrt(V->x*V->x + V->y*V->y + V->z*V->z);

  /* Normalization */
  V->x /= d; 
  V->y /= d; 
  V->z /= d; 
}

/*================== Draw galaxy (pne particle of galaxy) ==================*/
void DrawGalaxy(float psize)
{
  struct point v1, v2;
	float angg;
	
	/* Set particle orientation perpedicularly to the camera */
	v1.x = 0;
	v1.y = -1;
	v1.z = 0;
			
	v2.x = camd.x-cam.x;
	v2.y = camd.y-cam.y;
	v2.z = 0;

	angg = (float)acos(ScalarMult(&v1, &v2)/(VectorLen(&v1)*VectorLen(&v2)))*180/PI;
	glRotatef(-angg, 0, 0, 1);

	v1.x = 0;
	v1.y = -1;
	v1.z = 0;

	v2.x = 0;
	v2.y = camd.y-cam.y;
	v2.z = camd.z-cam.z;
	angg = (float)acos(ScalarMult(&v1, &v2)/(VectorLen(&v1)*VectorLen(&v2)))*180/PI;
	glRotatef(angg, 1, 0, 0);

  /* Draw polygon and particle texture mapping */
	glBegin(GL_QUADS);
  glTexCoord2f(0, 0); glVertex3f(-psize, 0, -psize);
  glTexCoord2f(1, 0); glVertex3f(-psize, 0, psize);
  glTexCoord2f(1, 1); glVertex3f(psize, 0, psize);
  glTexCoord2f(0, 1); glVertex3f(psize, 0, -psize);
  glEnd();

}

/*=================== Main Redraw Module ===================================*/
static void doRedraw()
{
  float a, b, ang1;

  int i;

  glClear(GL_COLOR_BUFFER_BIT);

	glLoadIdentity();
  
	glTranslatef(0, 0, -10);
  
	/* Set camera */
	gluLookAt(cam.x, cam.y, cam.z, camd.x, camd.y, camd.z, upv.x, upv.y, upv.z);

	/* Rotate galaxy */
	ang -= 0.1f;
	
	/* Draw inside part of galaxy (very condensed part) */
	glBindTexture(GL_TEXTURE_2D, 4);
  for (i=0; i<GALAXY_P; i++)
  {
		glColor3f(p1[i].r, p1[i].g, p1[i].b);
		glPushMatrix();
		a = p1[i].x*(float)cos(ang*PI/180)-p1[i].z*(float)sin(ang*PI/180);
    b = p1[i].x*(float)sin(ang*PI/180)+p1[i].z*(float)cos(ang*PI/180);
		glTranslatef(a, 0, b);
		DrawGalaxy(6);
		glPopMatrix();
	}
	
	/* Draw part of galaxy (normal condensed part) */
	glBindTexture(GL_TEXTURE_2D, 1);
  for (i=0; i<GALAXY_P; i++)
  {
		glColor4f(p2[i].r, p2[i].g, p2[i].b, 1);
		glPushMatrix();
		ang1 = ang*p2[i].speed;
		a = p2[i].x*(float)cos(ang1*PI/180)-p2[i].z*(float)sin(ang1*PI/180);
    b = p2[i].x*(float)sin(ang1*PI/180)+p2[i].z*(float)cos(ang1*PI/180);
		glTranslatef(a, p2[i].y, b);
		DrawGalaxy(2);
		glPopMatrix();
	}

	/* Draw part of galaxy (slightly condensed part) */
  for (i=0; i<GALAXY_P1; i++)
  {
		glColor4f(p3[i].r, p3[i].g, p3[i].b, 1);
		glPushMatrix();
		ang1 = ang*p3[i].speed;
		a = p3[i].x*(float)cos(ang1*PI/180)-p3[i].z*(float)sin(ang1*PI/180);
    b = p3[i].x*(float)sin(ang1*PI/180)+p3[i].z*(float)cos(ang1*PI/180);
		glTranslatef(a, p3[i].y, b);
		DrawGalaxy(1);
		glPopMatrix();
  }

	/* Draw foggly part of galaxy (simply polygon with texture) */
	glColor4f(1, 1, 1, 1);
	glBindTexture(GL_TEXTURE_2D, 3);
	glPushMatrix();
	glRotatef(-ang, 0, 1, 0);
	glBegin(GL_QUADS);
	glTexCoord2f(0, 0); glVertex3f(-60, 0, -60);
	glTexCoord2f(1, 0); glVertex3f(-60, 0, 60);
	glTexCoord2f(1, 1); glVertex3f(60, 0, 60);
	glTexCoord2f(0, 1); glVertex3f(60, 0, -60);
	glEnd();
	glPopMatrix();

	/* Draw background */
	glPushMatrix();
	/* Flip to projection mode */
	glMatrixMode(GL_PROJECTION);
  glPushMatrix();
	glLoadIdentity();
	/* Set new 2D projection */
	gluOrtho2D(-1, 1, -1, 1);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

	/* Draw background */
	glBindTexture(GL_TEXTURE_2D, 2);
	glColor4f(0.2f, 0.2f, 0.5f, 0.7f);
	glBegin(GL_QUADS);
	glTexCoord2f(0, 0); glVertex2f(-1, -1);
	glTexCoord2f(1, 0); glVertex2f(1, -1);
	glTexCoord2f(1, 1); glVertex2f(1, 1);
	glTexCoord2f(0, 1); glVertex2f(-1, 1);
	glEnd();

  /* Flip back to normal 3D mode */
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

  glFlush();
}