/**
Copyright (c) 2000 Brian Nenninger

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**/

#import <Foundation/Foundation.h>
#import <Appkit/Appkit.h>
#import <ScreenSaver/ScreenSaver.h>
#import "OCLifeHypergrid.h"
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#include <bsd/stdlib.h>
#include <bsd/math.h>

@interface LifeSaver : ScreenSaverView {
  BOOL _initedGL;
  NSOpenGLView *_view;
  OCLifeHypergrid *scene;
  
  double angle;
  double dist;
  GLUquadricObj *sphereQObj;
  
  int counter;
  BOOL inTransition;
  BOOL isPaused;
  
  // settings stored to preferences 
  int xcells, ycells, zcells; 
  int simspeed; // used to compute transitionFrames and fixedFrames
  int sphereDetail;
  BOOL useAllScreens;

  // values computed from above settings
  int transitionFrames;
  int fixedFrames;
  int numDimensions;
  double maxYZCoord;
  double maxXCoord;
  double spaceBetweenCells;
  double baseRadius;
  
  int rotateSpeed;
  int maxRotateSpeed;
  double rotateFactor;
  int gridRepetitionLimit;
  int gridRepetitionCounter;
  
  // configure sheet support
  NSWindow* configureSheet;
  id size2dField;
  id size3dField;
  id detailSlider;
  id speedSlider;
  id allScreensCheckbox;
}

-(BOOL)writePrefsDictionary:(NSDictionary *)dict;
-(NSDictionary *)readPrefsDictionary;

-(void)updateParameters;
-(BOOL)saveParameters;

// configure sheet methods
-(void)saveSettings:(id)sender;
-(void)cancelSettings:(id)sender;

-(void)increaseRotation;
-(void)decreaseRotation;
-(BOOL)shouldDraw;
-(void)initGL;
-(void)draw;
-(void)drawCellAtCoordinates:(OCHypergridCoordinates *)coords scaledBy:(double)scale;
-(void)dealloc;

@end
