/**
Copyright (c) 2000 Brian Nenninger

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**/

#import "OCHypergrid.h"
#import "OCHypergridCoordEnumerator.h"

@implementation OCHypergrid

-(id)initWithNumberOfDimensions:(int)numdim sizes:(int *)sizes {
  self = [super init];
  if (self) {
    _cgrid = hypergrid_create(numdim, sizes);
  }
  return self;
}

-(id)initWithDimensionsFromIntArray:(OCHypergridCoordinates *)array {
  self = [super init];
  if (self) {
    _cgrid = hypergrid_create([array count], [array cArray]);
  }
  return self;
}

-(id)initWithCHypergrid:(hypergrid *)gridptr {
  self = [super init];
  if (self) {
    _cgrid = hypergrid_copy(gridptr);
  }
  return self;
}

-(OCHypergrid *)copy {
  return [[OCHypergrid alloc] initWithCHypergrid:_cgrid];
}

-(int)numberOfDimensions {
  return _cgrid->num_dimensions;
}

-(gridvalue_t)valueAtCoordinates:(OCHypergridCoordinates *)coords {
  return hypergrid_get_value(_cgrid, [coords cArray]);
}

-(id)setValue:(gridvalue_t)value atCoordinates:(OCHypergridCoordinates *)coords {
  hypergrid_set_value(_cgrid, [coords cArray], value);
  return self;
}

-(int)nonzeroNeighborsOfCoordinates:(OCHypergridCoordinates *)coords {
  return hypergrid_neighbor_count(_cgrid, [coords cArray], 0, 0, 1);
}

-(int)nonzeroNeighborsOfCoordinates:(OCHypergridCoordinates *)coords wrapping:(BOOL)wrap {
  return hypergrid_neighbor_count(_cgrid, [coords cArray], 0, 0, wrap);
}

-(BOOL)isEqualToHypergrid:(OCHypergrid *)cmp {
  return hypergrid_equal([self cHypergrid], [cmp cHypergrid]);
}

-(NSEnumerator *)coordinateEnumerator {
  return [[[OCHypergridCoordEnumerator alloc] initWithHypergrid:self] autorelease];
}

-(hypergrid *)cHypergrid {
  return _cgrid;
}

-(void)dealloc {
  if (_cgrid) hypergrid_free(_cgrid);
  [super dealloc];
}

@end
