/* GLMorph.m created by epeyton on Thu 03-Dec-1998 */

#import "GLMorph.h"

//extern int mono;
//extern int smooth;
//extern int object;

#import "OpenGLmorph3d.c"

@implementation GLMorph	

- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL)preview
{
    self = [super initWithFrame:frameRect isPreview:preview];
    
    mp = (morph3dstruct *) malloc(sizeof (morph3dstruct));
    
    mp->isMono = 0;
    mp->isWireframe = 0;
    mp->step = SSRandomFloatBetween(0.0, 90.0);


    if (self) {
	NSOpenGLPixelFormatAttribute attribs[] =
        {	NSOpenGLPFAAccelerated,
		NSOpenGLPFADepthSize, 16,
		NSOpenGLPFAMinimumPolicy,
		NSOpenGLPFAClosestPolicy,
                0
        };
	
	NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
		
        _view = [[[NSOpenGLView alloc] initWithFrame:NSZeroRect pixelFormat:format] autorelease];
        [self addSubview:_view];
    }
    
    mp->isMono = [(NSNumber *)CFPreferencesCopyAppValue(@"GLMorphMono", @"EPScreenSavers") intValue];
    mp->VisibleSpikes = [(NSNumber *)CFPreferencesCopyAppValue(@"GLMorphUnshaded", @"EPScreenSavers") intValue];
    mp->object = [(NSNumber *)CFPreferencesCopyAppValue(@"GLMorphObjectType", @"EPScreenSavers") intValue];

    
    if (!mp->object) mp->object = 1;
    
    return self;
}

- (void)dealloc
{
    free(mp);
    [_view removeFromSuperview];
    [super dealloc];
}

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    [_view setFrameSize:newSize];
    _initedGL = NO;

}

- (void)animateOneFrame
{
    [[_view openGLContext] makeCurrentContext];
    if (!_initedGL) {
        pinit(mp);
        reshape(mp, (int)[self frame].size.width, (int)[self frame].size.height);

        _initedGL = YES;
    }
        draw_morph3d(mp);    
    return;
}


- (BOOL)hasConfigureSheet { return YES; }
- (NSWindow*)configureSheet { 

    [NSBundle loadNibNamed:@"GLMorph.nib" owner:self];
    [isMono setState:mp->isMono];
    [isUnshaded setState:mp->VisibleSpikes];
    [objectArray selectCellWithTag:mp->object];


    return window; 
}

- (void)closeSheet:(id)sender
{
    [NSApp endSheet:window];
}

- (void)updateSelection:sender
{
    mp->object = [objectArray selectedRow] + 1;
    mp->isMono = [isMono state];
    mp->VisibleSpikes = [isUnshaded state];
    
    CFPreferencesSetAppValue(@"GLMorphObjectType",[NSNumber numberWithInt:mp->object], @"EPScreenSavers");
    CFPreferencesSetAppValue(@"GLMorphMono",[NSNumber numberWithInt:mp->isMono], @"EPScreenSavers");
    CFPreferencesSetAppValue(@"GLMorphUnshaded",[NSNumber numberWithInt:mp->VisibleSpikes], @"EPScreenSavers");
    
    CFPreferencesAppSynchronize(@"EPScreenSavers");
    
    pinit(mp);

}

@end