/*	NAME:
		Particle.c

	DESCRIPTION:
		A particle effect fader for Setting Sun.

	___________________________________________________________________________
*/
//=============================================================================
//		Include Files.														 
//-----------------------------------------------------------------------------

#define SS_INCLUDE_OPENGL

#include <Appearance.h>
#include <Dialogs.h>
#include <Lists.h>
#include <ControlDefinitions.h>
#include <TextUtils.h>
#include <Types.h>
#include <Memory.h>
#include <Quickdraw.h>
#include <MacWindows.h>
#include <OSUtils.h>
#include <ToolUtils.h>
#include <stdio.h>
#include <gl.h>
#include <glu.h>
#include <agl.h>
#include <fp.h>

#include "SSAPI.h"





//=============================================================================
//		Constants
//-----------------------------------------------------------------------------
// Preferences

//=============================================================================
//		Global variables
//-----------------------------------------------------------------------------
// Preferences




//=============================================================================
//		Internal functions
//-----------------------------------------------------------------------------
//		UpdateState : Update our state.
//-----------------------------------------------------------------------------




//=============================================================================
//		External functions
//-----------------------------------------------------------------------------
//		FaderLoad : Called when the fader is loaded by Setting Sun.
//-----------------------------------------------------------------------------
#pragma mark -
OSErr FaderLoad(SSFaderInfoPtr faderInfo)
{	OSErr		theErr;



	// Validate our parameters
	SS_ASSERT_VALID_PTR(faderInfo);

	// Fill out the structure
	faderInfo->outAPIVersion  = kSSAPIVersion;						// API version we understand
	faderInfo->outDepth       = 16;									// Desired depth
	faderInfo->outClearColour = kColourBlack;						// Colour to clear to
	
	SetBit(faderInfo->outFaderRequests, kClearBackground);			// Clear the background

#if 0
	SetBit(faderInfo->outFaderRequests, kDebugInWindow);			// Run in a window?
#endif

#if 0
	SetBit(faderInfo->outFaderRequests, kDebugSoftwareRendering);	// Run without 3D hardware?
#endif

#if 0
	SetBit(faderInfo->outFaderRequests, kDebugShowFPS);				// Show FPS counter?
#endif

#if 1
	SetBit(faderInfo->outFaderRequests, kAllow3DFXRendering);		// Allow 3Dfx rendering?
#endif
	
	SS_ASSERT(TestBit(faderInfo->inInitedManagers, kInitialisedOpenGL));



	// Initialise our state
	SSClearMemory(particle, sizeof(particle));



	// Check the API versions are compatible
	theErr = SSValidateVersion(faderInfo);
	
	return(noErr);
}





//=============================================================================
//		FaderStart : Called when the fader is begining to fade.
//-----------------------------------------------------------------------------
OSErr FaderStart(GrafPtr faderPort, const Rect *faderRect)
{	
	int i,z;
	
	// Validate our parameters
	SS_ASSERT_VALID_PTR(faderPort);
	SS_ASSERT_VALID_PTR(faderRect);
	SS_ASSERT(faderRect->left < faderRect->right);
	SS_ASSERT(faderRect->top  < faderRect->bottom);
	
	viewRect.left = faderRect->left;
	viewRect.top = faderRect->top;
	viewRect.right = faderRect->right;
	viewRect.bottom = faderRect->bottom;
	
	width = viewRect.right - viewRect.left;
	height = viewRect.bottom - viewRect.top;
	
	ctx = SSGLContext_New(faderPort,faderRect,true,&kColourBlack);
	
	
	
	return(noErr);
}





//=============================================================================
//		FaderAnimate : Called repeatedly to draw each frame of the animation.
//-----------------------------------------------------------------------------
OSErr FaderAnimate(void)
{


	// Prepare the context
	aglSetCurrentContext(ctx);



	// Draw the particles


	return(noErr);
}





//=============================================================================
//		FaderUnload : Called when the fader is unloaded.
//-----------------------------------------------------------------------------
void FaderUnload(void)
{
	int i;
	
	for (i=0;i<num_particles;i++)
	{
		if (particle[i] != NULL)
			SSFree(particle[i]);
	}

	SSClearMemory(particle, sizeof(particle));
	SSGLContext_Dispose(&ctx);
}





//=============================================================================
//		FaderConfigure : Called when our configuration changes.
//-----------------------------------------------------------------------------
void FaderConfigure(Boolean areAnimating, UInt32 prefID)
{
	long new_number;
	int i;
	
	if (prefID == 5001)
	{
		SSGetPrefAsInteger(prefID,4,&new_number);
		
		if (new_number>num_particles)
			for (i=num_particles;i<new_number;i++)
				CreateRandomParticle(i);
		else if (new_number<num_particles)
			for (i=new_number;i<num_particles;i++)
				{
				if (particle[i] != NULL)
					{
					SSFree(particle[i]);
					particle[i] = NULL;
					}
				}
		
		num_particles = new_number;
	}
	else if (prefID == 5002)
		SSGetPrefAsBoolean(prefID,4,&fireworksMode);
		
	else if (prefID == 5003)
		SSGetPrefAsBoolean(prefID,4,&stereo);
		
	else if (prefID == 5004)
		SSGetPrefAsBoolean(prefID,4,&blueGlasses);

}
