#import "Pulsar.h"

#import "pulsar.c"


@implementation Pulsar
	
- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL)preview
{
    NSUserDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:@"EPPulsar"];

    ss = (screensaverstruct *) malloc(sizeof (screensaverstruct));
    
    self = [super initWithFrame:frameRect isPreview:preview];
    if (self) {
	NSOpenGLPixelFormatAttribute attribs[] =
        {	NSOpenGLPFAAccelerated,
		NSOpenGLPFADepthSize, 16,
		NSOpenGLPFAMinimumPolicy,
		NSOpenGLPFAClosestPolicy,
                0
        };
	
	NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
		
        _view = [[[NSOpenGLView alloc] initWithFrame:NSZeroRect pixelFormat:format] autorelease];
        [self addSubview:_view];
    }
    
    do_fog = [defaults integerForKey:@"GLPulsarFog"];
    do_blend = [defaults integerForKey:@"GLPulsarBlend"];
    do_antialias = [defaults integerForKey:@"GLPulsarAntialias"];
    do_wire = [defaults integerForKey:@"GLPulsarWire"];
    do_depth = [defaults integerForKey:@"GLPulsarDepth"];
    do_texture = [defaults integerForKey:@"GLPulsarTexture"];
    do_mipmap = [defaults integerForKey:@"GLPulsarMipmap"];
    do_light = [defaults integerForKey:@"GLPulsarLight"];

    if (!num_quads) num_quads = SSRandomInt() % 15;     

    return self;
}

- (void)dealloc
{
    [_view removeFromSuperview];
    free(ss);
    [super dealloc];
}

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    [_view setFrameSize:newSize];
    _initedGL = NO;

}

- (void)stopAnimation
{
    _initedGL = NO;
    [super stopAnimation];
}


- (void)animateOneFrame
{
    [[_view openGLContext] makeCurrentContext];
    if (!_initedGL) {
        reshape((int)[self frame].size.width, (int)[self frame].size.height);
        initializeGL((int)[self frame].size.width, (int)[self frame].size.height);

        _initedGL = YES;
    }
    draw_screensaver(ss);
    glFlush();
    return;
}

- (BOOL)hasConfigureSheet { return YES; }


- (void)closeSheet:(id)sender
{
    [NSApp endSheet:window];
}

- (void)updateSelections:(id)sender
{
    NSUserDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:@"EPPulsar"];

    do_fog = [useFog state];
    do_antialias = [useAntialias state];
    do_blend = [useBlend state];
    do_depth = [useDepth state];
    do_light = [useLighting state];
    do_mipmap = [useMipmap state];
    do_texture = [useTexture state];
    do_wire = [useWireframe state];

    [defaults setInteger:do_fog forKey:@"GLPulsarFog"];
    [defaults setInteger:do_antialias forKey:@"GLPulsarAntialias"];
    [defaults setInteger:do_blend forKey:@"GLPulsarBlend"];
    [defaults setInteger:do_depth forKey:@"GLPulsarDepth"];
    [defaults setInteger:do_light forKey:@"GLPulsarLight"];
    [defaults setInteger:do_mipmap forKey:@"GLPulsarMipmap"];
    [defaults setInteger:do_texture forKey:@"GLPulsarTexture"];
    [defaults setInteger:do_wire forKey:@"GLPulsarWire"];
    
    [defaults synchronize];
}

- (NSWindow*)configureSheet 
{ 

        [NSBundle loadNibNamed:@"GLPulsar.nib" owner:self];
        

        [planeSlider setIntValue:num_quads];
        [useFog setState:do_fog];
        [useAntialias setState:do_antialias];
        [useBlend setState:do_blend];
        [useDepth setState:do_depth];
        [useLighting setState:do_light];
        [useMipmap setState:do_mipmap];
        [useTexture setState:do_texture];
        [useWireframe setState:do_wire];

    return window;
}



@end
