#import "Tunnel.h"

#include "texture.c"

@implementation Tunnel
	
- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL)preview
{

    self = [super initWithFrame:frameRect isPreview:preview];
    if (self) {
	NSOpenGLPixelFormatAttribute attribs[] =
        {	NSOpenGLPFAAccelerated,
		NSOpenGLPFADepthSize, 16,
		NSOpenGLPFAMinimumPolicy,
		NSOpenGLPFAClosestPolicy,
                0
        };
	
	NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attribs] autorelease];
		
        _view = [[[NSOpenGLView alloc] initWithFrame:NSZeroRect pixelFormat:format] autorelease];
        [self addSubview:_view];
    }
    

    return self;
}

- (void)dealloc
{
    [_view removeFromSuperview];
    [super dealloc];
}

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    [_view setFrameSize:newSize];
    _initedGL = NO;

}

- (void)startAnimation
{
    [[_view openGLContext] makeCurrentContext];
    if (!_initedGL) {
                
        [self TunnelInit];
        [self LoadTexture];
        [self reshape:(int)[self frame].size.width :(int)[self frame].size.height];
    
        _initedGL = YES;
    }
    [super startAnimation];
}

- (void)stopAnimation
{
    _initedGL = NO;
    [super stopAnimation];

    return;
}

- (void)animateOneFrame
{
    [[_view openGLContext] makeCurrentContext];
	
    [self Draw];

    return;
}

- (BOOL)hasConfigureSheet { return NO; }
- (NSWindow*)configureSheet { return nil; }

- (void) TunnelInit
{
  GLfloat fogColor[4] = {0.6, 0.6, 0.0, 1.0};

  for (i=0; i<CONTP_TOTAL; i++) 
  {
    controlp[i].z = -i*DEPTH;
    controlp[i].xx = RADIUS*cos(45*PI/180);
    controlp[i].yy = RADIUS*sin(45*PI/180);
  }
  speed = 0.2;

  /* Enable fog */
  glEnable(GL_FOG);
  glFogi(GL_FOG_MODE, GL_EXP);
  glFogfv(GL_FOG_COLOR, fogColor);
  glFogf(GL_FOG_DENSITY, 0.08);
  glEnable(GL_CULL_FACE);

}

- (void) LoadTexture
{
  int texwid, texht;
  int texcomps;
  char *fn=[[[NSBundle bundleForClass:[Tunnel class]] pathForResource:@"texture" ofType:@"rgb"] cString];
 
  teximage = read_texture(fn, &texwid, &texht, &texcomps);
  if (!teximage)
  {
    printf("Sorry, can't read texture file...");
    exit(0);
  }
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, texwid, texht, 0, GL_RGBA, GL_UNSIGNED_BYTE, teximage);

  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 

  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
  glEnable(GL_TEXTURE_2D);  
} 

- (void) reshape:(int)width :(int)height
{
  glViewport(0, 0, width, height);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  glFrustum(-0.53, 0.53, -0.4, 0.4, 2, 50);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

- (void)Draw
{
  int total;
  
  glClear(GL_COLOR_BUFFER_BIT);

  glPushMatrix();

  for (i=0; i<CONTP_TOTAL-1; i++)
  {
    glBegin(GL_QUADS);
    glTexCoord2f(1, 0); glVertex3f(RADIUS, 0, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(controlp[i].xx, controlp[i].yy, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(controlp[i+1].xx, controlp[i+1].yy, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(RADIUS, 0, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(controlp[i].xx, controlp[i].yy, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(0, RADIUS, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(0, RADIUS, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(controlp[i+1].xx, controlp[i+1].yy, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(0, RADIUS, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(-controlp[i].xx, controlp[i].yy, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(-controlp[i+1].xx, controlp[i+1].yy, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(0, RADIUS, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(-controlp[i].xx, controlp[i].yy, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(-RADIUS, 0, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(-RADIUS, 0, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(-controlp[i+1].xx, controlp[i+1].yy, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(-RADIUS, 0, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(-controlp[i].xx, -controlp[i].yy, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(-controlp[i+1].xx, -controlp[i+1].yy, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(-RADIUS, 0, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(-controlp[i].xx, -controlp[i].yy, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(0, -RADIUS, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(0, -RADIUS, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(-controlp[i+1].xx, -controlp[i+1].yy, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(0, -RADIUS, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(controlp[i].xx, -controlp[i].yy, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(controlp[i+1].xx, -controlp[i+1].yy, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(0, -RADIUS, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(controlp[i].xx, -controlp[i].yy, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(RADIUS, 0, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(RADIUS, 0, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(controlp[i+1].xx, -controlp[i+1].yy, controlp[i+1].z);

    glEnd();   
  }
  glBegin(GL_QUADS);
  glVertex3f(-RADIUS, -RADIUS, -CONTP_TOTAL*2+15);
  glVertex3f(RADIUS, -RADIUS, -CONTP_TOTAL*2+15);
  glVertex3f(RADIUS, RADIUS, -CONTP_TOTAL*2+15);
  glVertex3f(-RADIUS, RADIUS, -CONTP_TOTAL*2+15);
  glEnd();

  total = 0;
  for (i=0; i<CONTP_TOTAL; i++)
  {
    controlp[i].z += speed;
    if (controlp[i].z > 0) total++;
  }

  /* Shift array (tunnel moving) */
  if (total > 0)
  {
    for (i=total; i<CONTP_TOTAL; i++)  
      controlp[i-total] = controlp[i];

    for (i=CONTP_TOTAL-total-1; i<CONTP_TOTAL; i++)
    {
      controlp[i].z = controlp[i-1].z-DEPTH;
      controlp[i].xx = RADIUS*cos(45*PI/180);
      controlp[i].yy = RADIUS*sin(45*PI/180);
    }
  }
  glPopMatrix();
  
  glFlush();

}

@end

