/*
--------------------------------------------------------------
                         OpenGL Tunnel Demo 3
  Author: Roman Podobedov
  Email: romka@ut.ee
  WEB: www.ut.ee/~romka
  Date: 26 July 2000
--------------------------------------------------------------
*/



void TunnelInit()
{
  GLfloat fogColor[4] = {0.6, 0.6, 0.0, 1.0};

  for (i=0; i<CONTP_TOTAL; i++) 
  {
    controlp[i].z = -i*DEPTH;
    controlp[i].xx = RADIUS*cos(45*PI/180);
    controlp[i].yy = RADIUS*sin(45*PI/180);
  }
  speed = 0.2;

  /* Enable fog */
  glEnable(GL_FOG);
  glFogi(GL_FOG_MODE, GL_EXP);
  glFogfv(GL_FOG_COLOR, fogColor);
  glFogf(GL_FOG_DENSITY, 0.08);
  glEnable(GL_CULL_FACE);

}

void LoadTexture()
{
  int texwid, texht;
  int texcomps;
  char fn[]="texture.rgb";
 
  teximage = read_texture(fn, &texwid, &texht, &texcomps);
  if (!teximage)
  {
    printf("Sorry, can't read texture file...");
    exit(0);
  }
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, texwid, texht, 0, GL_RGBA, GL_UNSIGNED_BYTE, teximage);

  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 

  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
  glEnable(GL_TEXTURE_2D);  
} 

void Reshape(int width, int height)
{
  glViewport(0, 0, width, height);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  glFrustum(-0.53, 0.53, -0.4, 0.4, 2, 50);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  WindW = width;
  WindH = height;
}

void Draw(void)
{
  int total;
  
  glClear(GL_COLOR_BUFFER_BIT);

  glPushMatrix();

  for (i=0; i<CONTP_TOTAL-1; i++)
  {
    glBegin(GL_QUADS);
    glTexCoord2f(1, 0); glVertex3f(RADIUS, 0, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(controlp[i].xx, controlp[i].yy, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(controlp[i+1].xx, controlp[i+1].yy, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(RADIUS, 0, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(controlp[i].xx, controlp[i].yy, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(0, RADIUS, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(0, RADIUS, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(controlp[i+1].xx, controlp[i+1].yy, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(0, RADIUS, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(-controlp[i].xx, controlp[i].yy, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(-controlp[i+1].xx, controlp[i+1].yy, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(0, RADIUS, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(-controlp[i].xx, controlp[i].yy, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(-RADIUS, 0, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(-RADIUS, 0, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(-controlp[i+1].xx, controlp[i+1].yy, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(-RADIUS, 0, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(-controlp[i].xx, -controlp[i].yy, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(-controlp[i+1].xx, -controlp[i+1].yy, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(-RADIUS, 0, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(-controlp[i].xx, -controlp[i].yy, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(0, -RADIUS, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(0, -RADIUS, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(-controlp[i+1].xx, -controlp[i+1].yy, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(0, -RADIUS, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(controlp[i].xx, -controlp[i].yy, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(controlp[i+1].xx, -controlp[i+1].yy, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(0, -RADIUS, controlp[i+1].z);

    glTexCoord2f(1, 0); glVertex3f(controlp[i].xx, -controlp[i].yy, controlp[i].z);
    glTexCoord2f(1, 1); glVertex3f(RADIUS, 0, controlp[i].z);
    glTexCoord2f(0, 1); glVertex3f(RADIUS, 0, controlp[i+1].z);
    glTexCoord2f(0, 0); glVertex3f(controlp[i+1].xx, -controlp[i+1].yy, controlp[i+1].z);

    glEnd();   
  }
  glBegin(GL_QUADS);
  glVertex3f(-RADIUS, -RADIUS, -CONTP_TOTAL*2+15);
  glVertex3f(RADIUS, -RADIUS, -CONTP_TOTAL*2+15);
  glVertex3f(RADIUS, RADIUS, -CONTP_TOTAL*2+15);
  glVertex3f(-RADIUS, RADIUS, -CONTP_TOTAL*2+15);
  glEnd();

  total = 0;
  for (i=0; i<CONTP_TOTAL; i++)
  {
    controlp[i].z += speed;
    if (controlp[i].z > 0) total++;
  }

  /* Shift array (tunnel moving) */
  if (total > 0)
  {
    for (i=total; i<CONTP_TOTAL; i++)  
      controlp[i-total] = controlp[i];

    for (i=CONTP_TOTAL-total-1; i<CONTP_TOTAL; i++)
    {
      controlp[i].z = controlp[i-1].z-DEPTH;
      controlp[i].xx = RADIUS*cos(45*PI/180);
      controlp[i].yy = RADIUS*sin(45*PI/180);
    }
  }
  glPopMatrix();
  
  glFlush();
  glutSwapBuffers();
}

static void Key(unsigned char key, int x, int y)
{
  switch (key)
  {
    case 27: exit(0);
             break;
    case 'a': speed += 0.01; if (speed > 1) speed = 1; break;
    case 'z': speed -= 0.01; if (speed < 0) speed = 0; break;
  }
}

void timf(int value)
{
  glutPostRedisplay();
  glutTimerFunc(20, timf, 0);
}


int main(int argc, char *argv[])
{

  WindW = 640;
  WindH = 480;

  srand((unsigned int)time(NULL));
  printf("Tunnel Demo\n");
  printf("Author: Roman Podobedov\n");
  printf("Email: romka@ut.ee\n");
  printf("WWW: www.ut.ee/~romka\n\n");
  printf("Control: \n");
  printf("a - increase speed\n");
  printf("z - decrease speed\n");
  glutInit(&argc, argv);
  glutInitWindowSize(WindW, WindH);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);

  (void)glutCreateWindow("Tunnel Demo 3");
  TunnelInit();
  LoadTexture();  

  glutReshapeFunc(Reshape);
  glutDisplayFunc(Draw);
  glutKeyboardFunc(Key);
  glClearColor(0, 0, 0, 0);

  glutTimerFunc(20, timf, 0);
  glutMainLoop();
  
  return 0;
}